-- Globals --
if not QuickCD then
    QuickCD = {}
end
QuickCD.Utils = {}

-- Methods --
function QuickCD.Utils.Round(num, idp) -- Rounds number to nearest whole integer. >= x.5 rounds up, < x.5 rounds down. Functions slightly differently for negative numbers.
    if num == nil then
        return nil
    end
    local mult = 10^(idp or 0)
    if num >= 0 then
        return math.floor(num * mult + .5) / mult
    else
        return math.ceil(num * mult - .5) / mult
    end
end

function QuickCD.Utils.DeepCopy(object) -- Copy's table so references don't screw stuff up.
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

function QuickCD.Utils.StrJoin(delimiter, list) -- Perl-like str.join functionality
  local len = #list
  if len == 0 then 
    return ""
  end
  local str = list[1]
  for i = 2, len do 
    string = string .. delimiter .. list[i] 
  end
  return str
end

function QuickCD.Utils.StrSplit(delimiter, text) -- Perl-like str.split functionality
  local list = {}
  local pos = 1
  if string.find("", delimiter, 1) then
    return "error"
  end
  while 1 do
    local first, last = string.find(text, delimiter, pos)
    if first then -- found?
      table.insert(list, string.sub(text, pos, first-1))
      pos = last+1
    else
      table.insert(list, string.sub(text, pos))
      break
    end
  end
  return list
end

function QuickCD.Utils.ToTimeStr(_time) -- Turns seconds integer into string. Adds 'h' to end for hours, 'm' to end for minutes. If bad input is sent, it sends out "0"
    if _time == nil then
        return " 0"
    end
    _time = tonumber(_time)
    if _time == nil then
        return " 0"
    end
    
    local timeType = 'seconds'
    if _time >= 3600 then
        _time = QuickCD.Utils.Round(_time  / 3600)
        timeType = 'hours'
    elseif _time >= 60 then
        _time = QuickCD.Utils.Round(_time  / 60)
        timeType = 'minutes'
    end
    
    local timeStr = ""
    if timeType == 'hours' then
        timeStr = string.format("%.0f", _time) .. 'h'
    elseif timeType == 'minutes' then
        timeStr = string.format("%.0f", _time) .. 'm'
    else
        timeStr = string.format("%.0f", _time)
    end
    
    local length = string.len(timeStr)
    if length == 1 then
        timeStr = " " .. timeStr
    end
    return timeStr
end